@@ -6,6 +6,8 @@ import pandas as pd
 
 import flox
 
+from .helpers import codes_for_resampling
+
 
 class Cohorts:
     """Time the core reduction function."""
@@ -113,6 +115,22 @@ class ERA5Dataset:
         )
 
 
+class ERA5Resampling(Cohorts):
+    def setup(self, *args, **kwargs):
+        super().__init__()
+        # nyears is number of years, adjust to make bigger,
+        # full dataset is 60-ish years.
+        nyears = 5
+        shape = (37, 721, 1440, nyears * 365 * 24)
+        chunks = (-1, -1, -1, 1)
+        time = pd.date_range("2001-01-01", periods=shape[-1], freq="h")
+
+        self.array = dask.array.random.random(shape, chunks=chunks)
+        self.by = codes_for_resampling(time, "D")
+        self.axis = (-1,)
+        self.expected = np.unique(self.by)
+
+
 class ERA5DayOfYear(ERA5Dataset, Cohorts):
     def setup(self, *args, **kwargs):
         super().__init__()
@@ -184,15 +202,6 @@ class ERA5Google(Cohorts):
         self.expected = pd.RangeIndex(self.by.max() + 1)
 
 
-def codes_for_resampling(group_as_index, freq):
-    s = pd.Series(np.arange(group_as_index.size), group_as_index)
-    grouped = s.groupby(pd.Grouper(freq=freq))
-    first_items = grouped.first()
-    counts = grouped.count()
-    codes = np.repeat(np.arange(len(first_items)), counts)
-    return codes
-
-
 class PerfectBlockwiseResampling(Cohorts):
     """Perfectly chunked for blockwise resampling."""
 
