@@ -109,11 +109,28 @@ len = partial(_len, func="len")
 nanlen = partial(_len, func="nanlen")
 
 
+def _var_std_wrapper(group_idx, array, engine, *, axis=-1, **kwargs):
+    # Attempt to increase numerical stability by subtracting the first element.
+    # https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
+    # Cast any unsigned types first
+    dtype = np.result_type(array, np.int8(-1) * array[0])
+    array = array.astype(dtype, copy=False)
+    first = _get_aggregate(engine).aggregate(group_idx, array, func="nanfirst", axis=axis)
+    array = array - first[..., group_idx]
+    return _get_aggregate(engine).aggregate(group_idx, array, axis=axis, **kwargs)
+
+
+var = partial(_var_std_wrapper, func="var")
+nanvar = partial(_var_std_wrapper, func="nanvar")
+std = partial(_var_std_wrapper, func="std")
+nanstd = partial(_var_std_wrapper, func="nanstd")
+
+
 def median(group_idx, array, engine, *, axis=-1, size=None, fill_value=None, dtype=None):
     return npg.aggregate_numpy.aggregate(
         group_idx,
         array,
-        func=partial(_casting_wrapper, np.median, dtype=array.dtype),
+        func=partial(_casting_wrapper, np.median, dtype=np.result_type(array.dtype)),
         axis=axis,
         size=size,
         fill_value=fill_value,
@@ -125,7 +142,7 @@ def nanmedian(group_idx, array, engine, *, axis=-1, size=None, fill_value=None,
     return npg.aggregate_numpy.aggregate(
         group_idx,
         array,
-        func=partial(_casting_wrapper, np.nanmedian, dtype=array.dtype),
+        func=partial(_casting_wrapper, np.nanmedian, dtype=np.result_type(array.dtype)),
         axis=axis,
         size=size,
         fill_value=fill_value,
@@ -137,7 +154,11 @@ def quantile(group_idx, array, engine, *, q, axis=-1, size=None, fill_value=None
     return npg.aggregate_numpy.aggregate(
         group_idx,
         array,
-        func=partial(_casting_wrapper, partial(np.quantile, q=q), dtype=array.dtype),
+        func=partial(
+            _casting_wrapper,
+            partial(np.quantile, q=q),
+            dtype=np.result_type(dtype, array.dtype),
+        ),
         axis=axis,
         size=size,
         fill_value=fill_value,
@@ -149,7 +170,11 @@ def nanquantile(group_idx, array, engine, *, q, axis=-1, size=None, fill_value=N
     return npg.aggregate_numpy.aggregate(
         group_idx,
         array,
-        func=partial(_casting_wrapper, partial(np.nanquantile, q=q), dtype=array.dtype),
+        func=partial(
+            _casting_wrapper,
+            partial(np.nanquantile, q=q),
+            dtype=np.result_type(dtype, array.dtype),
+        ),
         axis=axis,
         size=size,
         fill_value=fill_value,
@@ -163,7 +188,7 @@ def mode_(array, nan_policy, dtype):
     # npg splits `array` into object arrays for each group
     # scipy.stats.mode does not like that
     # here we cast back
-    return mode(array.astype(dtype, copy=False), nan_policy=nan_policy, axis=-1).mode
+    return mode(array.astype(dtype, copy=False), nan_policy=nan_policy, axis=-1, keepdims=True).mode
 
 
 def mode(group_idx, array, engine, *, axis=-1, size=None, fill_value=None, dtype=None):
