@@ -798,3 +798,16 @@ def test_groupby_preserve_dtype(reduction):
     expected = getattr(np, reduction)(ds.test.data, axis=0).dtype
 
     assert actual == expected
+
+
+@requires_dask
+def test_resample_first_last_empty():
+    with xr.set_options(use_flox=True), dask.config.set(scheduler="processes"):
+        arr = xr.DataArray(
+            np.nan,
+            coords={
+                "date": pd.to_datetime(["2025-03-24", "2025-06-23"]),
+            },
+            dims=["date"],
+        ).chunk(date=(1, 1))
+        arr.resample(date="QE").last().compute()
